def get_task_name(options):
    """
    Given a dictionary of command options, return the name of the task
    :param options:
    :return:
    """
    options_dict = dict(options)
    return [task for task in [key for key in options_dict] if str(options_dict[task]) == 'True'][0]



import logging

from django.db.models.signals import post_save
from django.dispatch import receiver

from sidekick_tasks.models import Task
from sidekick.services.cron_files import CronService

logger = logging.getLogger(__name__)


@receiver(post_save, sender=Task)
def task_post_save(sender, instance, **kwargs):
    """Methods to be run on the post save of the Task model """
    try:
        CronService().generate_cron_tasks()
    except Exception as e:
        logger.error(msg='Failed to generate cron task for {} due to {}'.format(instance, e))



import logging

from sidekick_tasks.models import Task
from sidekick_tasks.settings import SIDEKICK_REGISTERED_APPS, ENVIRONMENT, DJANGO_PATH, CRON_PATH

logger = logging.getLogger(__name__)


class CronService:
    """Service to create or re-write the cron files """

    def __init__(self):
        self.environment = ENVIRONMENT
        self.joiner = "&& cd"
        self.django_path = DJANGO_PATH

    def write_cron_file(self, tasks):
        """For each task in list, write to the cron file. """

        with open(CRON_PATH, 'w+') as ws_cron_file:
            for task in tasks:
                ws_cron_file.write(
                    "# {name}\n"
                    "{schedule} {environment} {joiner} {django_path} {task}\n\n".format(
                        name=task.name,
                        environment=self.environment,
                        joiner=self.joiner,
                        django_path=self.django_path,
                        schedule=task.cron_schedule.schedule(),
                        task=task.registered_task.task_name)
                )

    def generate_cron_tasks(self):
        """Create a new cron file on the post save of a Registered Task"""
        try:
            tasks = Task.objects.filter(enabled=True)
            self.write_cron_file(tasks=tasks)
            logger.info(msg='Cron tasks successfully created.')
        except Exception as e:
            logger.error(msg='Failed to write cron tasks due to {}'.format(e))



from django.apps import AppConfig


class SidekickConfig(AppConfig):
    name = 'sidekick'

    def ready(self):
        import sidekick.signals.post_save

        self.register_tasks()

    @staticmethod
    def register_tasks():
        """
        For each app listed in sidekick_tasks.settings SIDEKICK_REGISTERED_APPS import the task to trigger the decorator
        :return:
        """
        import importlib
        from sidekick_tasks.settings import SIDEKICK_REGISTERED_APPS

        for app in SIDEKICK_REGISTERED_APPS:
            app_task = app + '.tasks'
            importlib.import_module("%s" % app_task)



import os
import logging
import importlib

from sidekick_tasks.settings import LOCK_PATH

logger = logging.getLogger(__name__)


class CronTask:
    """Helpers for cron tasks """

    def __init__(self, task_name, app):
        """Initialise variables and paths for CronTask class

        :param task_name: Name of task to run, will be displayed in lock files
        """
        self.app = app
        self.task_name = task_name
        self.lock_path = LOCK_PATH
        self.lock_file = os.path.join(self.lock_path, '{}.lock'.format(self.task_name))

    def run(self):
        """Try to run management function.

        If lock file does not exist, run task, delete lock file.
        """
        if not self.lock_file_exists():
            self.create_lock_file()
            try:
                app_task = self.app + '.tasks'
                importlib.import_module("%s" % app_task)
                # call the function using the getattr function
                getattr(importlib.import_module("%s" % app_task), self.task_name)()
                logger.info(msg='Successfully ran {}'.format(self.task_name))
            except Exception as e:
                logger.error(msg=e)
            self.delete_lock_file()

    def create_lock_file(self):
        """
        Create lock file before task is run at your specified path eg. /var/lock/project/task_name.lock
        """
        os.makedirs(self.lock_path, exist_ok=True)
        with open(self.lock_file, 'w+'):
            return False

    def delete_lock_file(self):
        """
        Delete lock file after task is run.
        """
        if os.path.exists(self.lock_file):
            os.remove(self.lock_file)

    def lock_file_exists(self):
        """Check if lock file exists for task.

        If it exists, notify us as that the task is starting again before finishing.
        Feel free to amend this and send yourself emails rather than logging

        :return: True if lock file exists
        """
        if os.path.isfile(self.lock_file):
            logger.info(msg="Lock file already exists for {} so the task did not run again".format(self.lock_file))
            return True
        return False



import inspect
from django.db import connection

# This is here in case you have tasks decorated before running migrations
# It will just return the original function if the table does not exist

if 'sidekick_tasks_registeredtask' in connection.introspection.table_names():

    def sidekick_task(fn):
        """
        Strip back any unnecessary text so that you can grab the task name and the app
        it came from in order to create a RegisteredTask instance in the correct format
        :param fn: Any function you want to run as a task
        :return: The original function
        """
        from sidekick_tasks.models import RegisteredTask

        src = inspect.getsource(fn)
        module_name = inspect.getmodule(fn).__name__.split('.')[0]
        task_string = src.split('(')[0][17:].replace('f ', '{} --'.format(module_name))
        try:
            RegisteredTask.objects.get(task_name=task_string)
        except RegisteredTask.DoesNotExist:
            st = RegisteredTask()
            st.task_name = task_string
            st.save()
        return fn

else:
    def sidekick_task(fn):
        pass
